import math

from sqlalchemy import or_

from flask import (request, render_template, redirect, url_for,
    jsonify, abort, current_app, flash)
from flask_login import current_user, login_required
from flask_babel import gettext, pgettext

from fardel.core.auth.models import User, Group, Permission
from fardel.ext import db

from .. import mod, staff_required, admin_required, permission_required


#########
# USERS #
#########

@mod.route('/auth/users/list/')
@permission_required("can_get_users")
@staff_required
@login_required
def users_list():
    page = request.args.get('page', type=int, default=1)
    per_page = request.args.get('per_page', type=int, default=40)
    query = User.query.filter_by(is_staff=False, is_admin=False)
    pages = math.ceil(query.count() / per_page)
    users = query.paginate(
        page=page, per_page=per_page, error_out=False).items
    return render_template('auth/users_list.html',
        users=users, pages=pages, page=page)


@mod.route('/auth/users/edit/<int:user_id>/', methods=['POST', 'GET'])
@admin_required
@staff_required
@login_required
def users_edit(user_id):
    edditable_attrs = [
        'email', 'password', 'first_name',
        'last_name', 'group_id', 'confirmed',
        'is_admin', 'is_staff'
    ]
    user = User.query.filter_by(id=user_id).first_or_404()
    if request.method == "POST":
        email = request.form.get('email')
        first_name = request.form.get('first_name')
        last_name = request.form.get('last_name')
        password = request.form.get('password')
        confirmed = request.form.get('confirmed', type=bool)
        is_admin = request.form.get('is_admin', type=bool)
        is_staff = request.form.get('is_staff', type=bool)
        group_id = request.form.get('group_id')
        
        if not email or not first_name or not last_name:
            flash('gettext(email, first name, last name and password fields can not be empty!)', 'error')    
        user._email = email
        user.first_name = first_name
        user.last_name = last_name
        user.group_id = group_id

        if password and password != "":
            user.password = password

        user.confirmed = confirmed
        user.is_admin = is_admin
        user.is_staff = is_staff
        db.session.commit()
        if user.is_admin:
            return redirect(url_for('panel.staffs_list'))
            
        return redirect(url_for('panel.users_list'))
        
    groups = Group.query.all()
    return render_template('auth/users_form.html', user=user, groups=groups)

@mod.route('/auth/users/create/', methods=['POST', 'GET'])
@admin_required
@staff_required
@login_required
def users_create():
    if request.method == "POST":
        email = request.form.get('email')
        if User.query.filter_by(_email=email).first():
            flash(gettext('user with this email already exists.'), 'error')
            return redirect(url_for('panel.users_create'))

        first_name = request.form.get('first_name')
        last_name = request.form.get('last_name')
        password = request.form.get('password')
        confirmed = request.form.get('confirmed', type=bool)
        is_admin = request.form.get('is_admin', type=bool)
        group_id = request.form.get('group_id')

        if not email or not first_name or not last_name:
            flash(gettext('email, first name, last name and password fields can not be empty!'), 'error')    
            return redirect(url_for('panel.users_create'))

        user = User(_email=email, first_name=first_name, last_name=last_name, password=password,
                    confirmed=confirmed, is_admin=is_admin, group_id=group_id)    

        db.session.add(user)
        db.session.commit()
        flash(gettext('user successfully created'), 'success')
        return redirect(url_for("panel.users_list"))
    groups = Group.query.all()
    return render_template('auth/users_form.html', groups=groups)

@mod.route('/auth/users/delete/<int:user_id>/')
@admin_required
@staff_required
@login_required
def users_delete(user_id):
    abort(404)

################
# STAFFMEMBERS #
################

@mod.route('/auth/staffs/list/')
@permission_required("can_get_users")
@admin_required
@staff_required
@login_required
def staffs_list():
    page = request.args.get('page', type=int, default=1)
    per_page = request.args.get('per_page', type=int, default=40)
    query = User.query.filter(or_(User.is_staff==True, User.is_admin==True))
    pages = math.ceil(query.count() / per_page)
    users = query.paginate(
        page=page, per_page=per_page, error_out=False).items
    return render_template('auth/users_list.html',
        users=users, pages=pages, page=page)

@mod.route('/auth/staffs/edit/<int:user_id>/')
@admin_required
@staff_required
@login_required
def staffs_edit(user_id):
    abort(404)

@mod.route('/auth/staffs/create/')
@admin_required
@staff_required
@login_required
def staffs_create():
    abort(404)

###############
# PERMISSIONS #
###############

@mod.route('/auth/permissions/list/')
@permission_required('can_get_permissions')
@staff_required
@login_required
def permissions_get():
    return jsonify({
        'permissions': [obj.dict() for obj in 
            Permission.query.all()]
    })

###########
# GRPOUPS #
###########

@mod.route('/auth/groups/list/')
@permission_required('can_get_groups')
@staff_required
@login_required
def groups_list():
    abort(404)
    if group_id:
        g = Group.query.filter_by(id=group_id).first()
        if not g:
            return {"message": "Group not found"}, 404
        return {'group': g.dict()}

    return {
        "groups": [obj.dict() for obj in 
            Group.query.all()]
    }

@mod.route('/auth/permissions/create/')
@admin_required
@staff_required
@login_required
def groups_create(self):
    abort(404)



import sqlalchemy
from flask import jsonify, request

from flask_restful import Resource, abort
from flask_babel import gettext

from fardel.ext import db

__all__ = (
    'BaseResource', 'GetBaseResource'
)


class BaseResource(Resource):

    def bad_request(self):
        return {'message': gettext('Invalid form submitted')}, 400

    def check_data(self, data, *or_requires):
        for requires in or_requires:
            satisfied = True

            if not isinstance(requires, list):
                raise Exception("or_requires are list each item not %s" % type(requires))

            for r in requires:
                if r not in data:
                    satisfied = False
                if not data[r]:
                    satisfied = False

            if satisfied:
                return True

        return False


class GetBaseResource(BaseResource):
    resource_class = None

    def check_implemented(self):
        if self.resource_class == None:
            raise NotImplementedError("resource_class have to be assigned")

    def obj_id_required(self):
        return {"message": gettext("obj_id must be provided")}, 422

    def get(self, obj_id=None):
        self.check_implemented()

        if obj_id:
            u = self.resource_class.query.filter_by(id=obj_id).first()
            if not u:
                return {"message": "%s not found" % self.resource_class.__name__}, 404
            return {self.resource_class.__name__.lower(): u.dict()}
        resource_name = "%ss" % self.resource_class.__name__.lower()
        page = request.args.get('page', type=int, default=1)
        return {
            resource_name: [obj.dict() for obj in
                            self.resource_class.query.paginate(
                page=page, per_page=32, error_out=False).items]
        }


class PostBaseResource(GetBaseResource):
    required_to_create = None
    optional_fields = ()

    def check_implemented(self):
        super(PostBaseResource, self).check_implemented()
        if self.required_to_create == None:
            raise NotImplementedError("required_to_create have to be assigned")

    def post(self, obj_id=None):
        self.check_implemented()

        fields = {}
        data = request.get_json()
        for field in self.required_to_create:
            if not data.get(field):
                return {'message': '%s must be provided.' % field}, 422
            fields[field] = data[field]

        for field in self.optional_fields:
            fields[field] = data.get(field)

        obj = self.resource_class(**fields)
        db.session.add(obj)
        try:
            db.session.commit()
        except sqlalchemy.exc.IntegrityError:
            return {"message": "%s already exists" % self.resource_class.__name__}, 422
        return {
            'message': "%s successfully added" % self.resource_class.__name__,
            self.resource_class.__name__.lower(): obj.dict()
        }


class DeleteBaseResource(GetBaseResource):
    def delete(self, obj_id=None):
        self.check_implemented()
        if not obj_id:
            abort(403)

        deleteds = self.resource_class.query.filter_by(id=obj_id).delete()
        db.session.commit()
        if deleteds == 1:
            return {"message": "%s successfully deleted" % self.resource_class.__name__}
        elif deleteds > 1:
            return {
                "message": "%ss successfully deleted, count: %d" % (
                    self.resource_class.__name__, deleteds
                )}
        return {"message": "No %s deleted" % self.resource_class.__name__.lower()}, 404



from flask_script import Manager, Command
from flask_migrate import Migrate, MigrateCommand

from fardel import Fardel
from fardel.ext import db

from fardel.core.auth.models import User


def create_admin(email, password):
    u = User(email=email, password=password, is_admin=True)
    db.session.add(u)
    db.session.commit()
    print("Successfull created an admin.")


class FardelManager():
    def __init__(self, fardel: Fardel):
        self.fardel = fardel

        migrate = Migrate(fardel.app, db)
        self.flask_manager = Manager(fardel.app)
        self.flask_manager.add_command('db', MigrateCommand)

        self.register_commands()

    def register_commands(self):
        self.flask_manager.add_command("create_admin", Command(create_admin))

    def run(self):
        self.flask_manager.run()



"""
Objects
=======

Object are json objects.

1. Permission
    :name: Name of the permission to display it.
    :code_name: the code for the permission.

2. Group
    :id: ID of the group in database.
    :name: Name of the group to display it.
    :permissions: List of permission object.

3. User
    :id: ID of the user in database.
    :first_name: First name of the user.
    :last_name: Last name of the user.
    :email: email of the user.


4. Access
    At login API, if the user is staffmember or admin we have such an object.

    :group: The group object which user is member of it.
    :is_admin: If the user is admin then its true.
    :is_staff: If the user is staffmember then its true.  
    

Errors
======

If a token needs to be refreshed:

:status_code: 401
:response:
    .. code-block:: json
        
        {"message": "Fresh token required"}

Invalid token:

:status_code: 422
:response:
    .. code-block:: json

        {"message": "reason"}

Expired token:

:status_code: 401
:response:
    .. code-block:: json

        {"message": "Token has expired"}

Revoked token:

:status_code: 401
:response:
    .. code-block:: json

        {"message":"Token has been revoked"}
"""
import time

from sqlalchemy import or_
from flask import render_template, redirect, url_for, jsonify, request, make_response, current_app

from fardel.core.rest import create_api, abort, Resource
from flask_jwt_extended import (
    create_access_token, create_refresh_token,
    jwt_required, jwt_refresh_token_required,
    get_jwt_identity, get_raw_jwt, current_user,
    jwt_optional
)

from fardel.ext import jwt, db
from fardel.core.base import BaseResource
from fardel.core import email as email_pkg

from . import mod
from .models import *


auth_api = create_api(mod)


@mod.before_app_first_request
def create_permissions():
    setup_permissions()


def rest_resource(resource_cls):
    """ Decorator for adding resources to Api App """
    auth_api.add_resource(resource_cls, *resource_cls.endpoints)
    return resource_cls


@rest_resource
class RegistrationApi(BaseResource):
    """
    :URL: ``/api/auth/register/``
    """
    endpoints = ['/register/']

    def already_exists(self):
        return {
            'message': 'A user with this email already exists.'
        }, 409

    def post(self):
        """
        :required arguments:
            * email
            * password
        :optional arguments:
            * first_name
            * last_name
        :response:
            .. code-block:: json

               {
                  "message":"Successfully registered",
                  "access_token":"access_token",
                  "refresh_token":"refresh_token"
               }

        :errors:
            if email or password does not provided:

            :status_code: 400
            :response:
                .. code-block:: json

                    {"message":"Invalid form submitted"}

            If email already exists:

            :status_code: 409
            :response:
                .. code-block:: json

                    {"message": "A user with this email already exists."}
        """
        data = request.get_json()
        if not self.check_data(data, ['password', 'email'], ['password', 'phone_number']):
            return self.bad_request()

        password = data['password']
        email = data.get('email')
        phone_number = data.get('phone_number')
        first_name = data.get('first_name')
        last_name = data.get('last_name')

        if User.query.filter_by(_email=email).first():
            return self.already_exists()

        u = User(password=password, _email=email,
                 first_name=first_name, last_name=last_name)

        db.session.add(u)
        db.session.commit()

        if current_app.config['EMAIL_ACTIVE']:
            email = u.generate_registration_email()
            email_pkg.send_email(email)

        access_token = create_access_token(identity=u.email)
        refresh_token = create_refresh_token(identity=u.email)
        return {
            'message': 'Successfully registered',
            'access_token': access_token,
            'refresh_token': refresh_token
        }


@rest_resource
class LoginApi(BaseResource):
    """
    :URL: ``/api/auth/login/``
    """
    endpoints = ['/login/']

    def post(self):
        """
        :required arguments:
            * email
            * password
        :response:
            .. code-block:: python

               {
                  "message":"Successfully registered",
                  "access_token":"access_token",
                  "refresh_token":"refresh_token",
                  "access": AccessObject
               }

        :errors:
            if email or password does not provided:

            :status_code: 400
            :response:
                .. code-block:: json

                    {"message":"Invalid form submitted"}

            If email or password is not correct:

            :status_code: 401
            :response:
                .. code-block:: json

                    {"message":"Username or password is not correct"}
        """
        data = request.get_json()
        if not self.check_data(data, ['email', 'password']):
            return self.bad_request()

        password = data['password']
        email = data['email']

        u = User.query.filter_by(_email=email).scalar()
        if u and u.check_password(password):
            access_token = create_access_token(identity=u.email)
            refresh_token = create_refresh_token(identity=u.email)
            obj = {
                'message': 'Successfully logined',
                'access_token': access_token,
                'refresh_token': refresh_token,
            }
            access = u.access_dict()
            if access:
                obj['access'] = access
            return obj
        return {
            'message': 'Username or password is not correct'
        }, 401


@rest_resource
class LogoutApi(BaseResource):
    """
    :URL: ``/api/auth/logout/``
    """
    endpoints = ['/logout/']
    method_decorators = [jwt_required]

    def post(self):
        """
        * Authorization header containing access token is required

        :status_code: 200
        :response:
            .. code-block:: json

                {
                    "message": "Access token has been revoked"
                }
        """
        jti = get_raw_jwt()['jti']
        revoked_token = RevokedToken(jti=jti)
        revoked_token.add()
        return {'message': 'Access token has been revoked'}


@rest_resource
class LogoutRefreshApi(BaseResource):
    """
    :URL: ``/api/auth/logout-refresh/``
    """
    endpoints = ['/logout-refresh/']
    method_decorators = [jwt_refresh_token_required]

    def post(self):
        """
        * Authorization header containing refresh token is required

        :status_code: 200
        :response:
            .. code-block:: json

                {
                    "message": "Refresh token has been revoked"
                }
        """
        jti = get_raw_jwt()['jti']
        revoked_token = RevokedToken(jti=jti)
        revoked_token.add()
        return {'message': 'Refresh token has been revoked'}


@rest_resource
class RefreshTokenApi(BaseResource):
    """
    :URL: ``/api/auth/refresh-token/``
    """
    endpoints = ['/refresh-token/']
    method_decorators = [jwt_refresh_token_required]

    def post(self):
        """
        * Authorization header containing refresh token is required

        :status_code: 200
        :response:
            .. code-block:: json

                {
                    "access_token": "access_token"
                }
        """
        current_user = get_jwt_identity()
        access_token = create_access_token(identity=current_user)
        return {'access_token': access_token}


@rest_resource
class ProfileApi(BaseResource):
    """
    :URL: ``/api/auth/profile/``
    """
    endpoints = ['/profile/']
    method_decorators = {
        'get': [jwt_required],
        'put': [jwt_required]
    }

    def get(self):
        """
        * Authorization header containing access token is required

        :status_code: 200
        :response:
            .. code-block:: python

                {
                    "user": UserObject
                }
        """
        return {"user": current_user.dict()}

    def put(self):
        """
        * Authorization header containing refresh token is required

        :status_code: 200
        :response:
            .. code-block:: python

                {
                    "message": "Profile successfully updated"
                    "user": UserObject
                }
        """
        data = request.get_json()
        fields = {
            "first_name": data.get('first_name'),
            "last_name": data.get('last_name'),
            "email": data.get('email'),
            "password": data.get('password'),
        }
        for field in fields:
            if data.get(field):
                setattr(current_user, field, data[field])

        db.session.commit()
        return {"message": "Profile successfully updated",
                'user': current_user.dict()}


@rest_resource
class ConfirmEmailAPI(BaseResource):
    """
    :URL: ``/api/auth/confirm/email/`, `/api/auth/confirm/email/<token>/``
    """
    endpoints = ['/confirm/email/', '/confirm/email/<token>/']

    def count_active_tokens(self, tokens):
        count = 0
        for t in tokens:
            if t['expire_at'] > time.time():
                count += 1
        return count

    @jwt_required
    def get(self):
        if current_user.tokens and self.count_active_tokens(current_user.tokens) > 2:
            return {"message": "You have two active tokens"}, 403

        email = current_user.generate_registration_email()
        email_pkg.send_email(email)
        return {"message": "an email sent to you provided by a token"}

    @jwt_required
    def patch(self, token):
        is_confirmed = current_user.confirm_email_with_token(token)
        if is_confirmed:
            return {"message": "email confirmed successfully"}
        return {"message": "token is not valid"}, 404



from flask import redirect, url_for, render_template, abort, request, flash
from flask_login import (current_user, login_user,
	logout_user, login_required, logout_user)
from flask_babel import gettext

from fardel.core.auth.models import User

from .. import mod

from . import auth
from . import media

from ..sidebar import panel_sidebar, Section, Link, ChildLink


@mod.before_app_first_request
def add_home_section():
	section = Section('')
	section.add_link(Link('fa fa-home', gettext('home'), url_for('panel.home')))
	panel_sidebar.add_section(section)


@mod.before_app_first_request
def add_auth_section():
	section = Section(gettext("Authentication"))
	link = Link("fa fa-user", gettext("Users"))
	link.add_child(ChildLink(gettext("Users"), url_for('panel.users_list')))
	link.add_child(ChildLink(gettext("Staffmembers"), url_for('panel.staffs_list')))
	section.add_link(link)

	# link = Link("fa fa-users", gettext("Groups"), url_for('panel.groups_list'))
	# section.add_link(link)
	panel_sidebar.add_section(section)


# @mod.before_app_first_request
# def add_media_section():
# 	section = Section(gettext("Uploads"))
# 	link = Link("fa fa-file", gettext("Files"))
# 	link.add_child(ChildLink(gettext("Files"), "#"))
# 	# link.add_child(ChildLink("آلبوم عکس", "#"))
# 	section.add_link(link)

# 	panel_sidebar.add_section(section)


@mod.route('/login/', methods=['POST', 'GET'])
def login():
	if current_user.is_authenticated:
		return redirect(url_for('panel.home'))
		
	if request.method == "POST":
		email = request.form.get('email')
		password = request.form.get('password')

		u = User.query.filter_by(_email=email).first()
		if u and (u.is_staff or u.is_admin) and u.check_password(password):
			login_user(u)
			return redirect(url_for('panel.home'))
		flash(gettext('Email or password is wrong...'))
	return render_template('login.html')

@mod.route('/logout/')
@login_required
def logout():
	logout_user()
	return redirect('/')

@mod.route('/')
def index():
	if not current_user.is_authenticated:
		abort(404)
	return redirect(url_for('panel.home'))

@mod.route('/home/')
@login_required
def home():
	return render_template('dashboard.html')


"""

Objects
=======

1. File
    :name: 
    :url:
    :path:


2. Directory
    :name: Name of the directory
    :path: Path of the folder


"""

import os

from flask import request, current_app
from flask_jwt_extended import jwt_required

from fardel.core.rest import create_api, abort, Resource
from fardel.ext import db

from .. import mod, staff_required_rest, permission_required
from ...base import BaseResource
from ...media.models import File


panel_media_api = create_api(mod)

panel_decorators = [staff_required_rest, jwt_required]

def rest_resource(resource_cls):
    """ Decorator for adding resources to Api App """
    panel_media_api.add_resource(resource_cls, *resource_cls.endpoints)
    return resource_cls


def is_safe_path(basedir, path, follow_symlinks=True):
    # resolves symbolic links
    if follow_symlinks:
        return os.path.realpath(path).startswith(basedir)
    return os.path.abspath(path).startswith(basedir)


@rest_resource
class FileApi(Resource):
    """
    :URL: ``/api/panel/files/``
    """
    endpoints = ['/files/']
    method_decorators = {
        'get': [permission_required('can_see_directory')] + panel_decorators,
        'post': [permission_required('can_create_files')] + panel_decorators,
        'delete': [permission_required('can_delete_files')] + panel_decorators,
    }
    def get(self):
        """
        Directory listing of upload folder
        
        :user permission required: can_see_directory

        """
        uploads_folder = current_app.config['UPLOAD_FOLDER']
        path = request.args.get('path', '')
        lookup_folder = uploads_folder / path
        if not os.path.exists(str(os.getcwd() / lookup_folder)):
            return {"message": "No such a folder"}, 404

        if is_safe_path(str(os.getcwd() / uploads_folder), str(lookup_folder)):
            response = {'files': [], 'directories':[]}
            for obj in os.listdir(lookup_folder):
                if os.path.isdir(lookup_folder / obj):
                    response['directories'].append({'name':obj, 'path':os.path.join(path, obj)})
                else:
                    file = File(path, file_name=obj)
                    file = {
                        'name':obj,
                        'url':file.url,
                        'path':os.path.join(path, obj)
                    }
                    response['files'].append(file)
            return response
        return {"message":"Invalid path"}, 422

    def post(self):
        """ To create file """
        file = request.files.get('file')
        path = request.form.get('path', '')
        if not file:
            return {"message":"No file to upload"}, 422

        uploads_folder = current_app.config['UPLOAD_FOLDER']
        lookup_folder = uploads_folder / path
        if is_safe_path(str(os.getcwd() / lookup_folder), str(lookup_folder)):
            file = File(path, file=file)
            file.save()
            return {'message':'File saved successfully', 'file':{"url":file.url}}
        return {"message":"Invalid path"}, 422

    def delete(self):
        """ To delete file """
        data = request.get_json()
        if not data.get('url'):
            return {"message": "No file url specified"}, 422

        url = data['url']
        path_to_file = current_app.config['UPLOAD_FOLDER'] / url.replace('/uploads/', '', 1)

        if os.path.isdir(path_to_file):
            try:
                os.rmdir(path_to_file)
            except:
                return {"message":"Directory must be empty"}, 422
            return {"message":"File deleted successfully"}
            
        if os.path.isfile(path_to_file):
            os.remove(path_to_file)
            return {"message":"File deleted successfully"}

        return {"message":"File not found"}, 404


@rest_resource
class ImageAlbumApi(BaseResource):
    """
    :URL: ``/api/panel/image_album/``    
    """
    endpoints = ['/image_album/']
    method_decorators = {
        'get': [permission_required('can_see_directory')] + panel_decorators,
        'post': [permission_required('can_create_files')] + panel_decorators,
        'delete': [permission_required('can_delete_files')] + panel_decorators,
    }
    def get(self):
        """ Directory listing of upload folder """
        uploads_folder = current_app.config['UPLOAD_FOLDER']
        path = 'images'
        lookup_folder = uploads_folder / path
        response = {'images': []}
        for obj in os.listdir(lookup_folder):
            if not os.path.isdir(lookup_folder / obj):
                file = File(path, file_name=obj)
                file = {'name':obj, 'url':file.url}
                response['images'].append(file)
        return response

    def post(self):
        """ To create file """
        file = request.files.get('file')
        path = 'images'
        if not file:
            return {"message":"No file to upload"}, 422

        uploads_folder = current_app.config['UPLOAD_FOLDER']
        lookup_folder = uploads_folder / path
        file = File(path, file=file)
        file.save()
        return {'message':'Image saved successfully', 'file':{"url":file.url}}

    def delete(self):
        """ To delete file """
        data = request.get_json()
        if not data.get('url'):
            return {"message": "No file url specified"}, 422

        url = data['url']
        path_to_file = current_app.config['UPLOAD_FOLDER'] / url.replace('/uploads', 'images', 1)
            
        if os.path.isfile(path_to_file):
            os.remove(path_to_file)
            return {"message":"Image deleted successfully"}

        return {"message":"Image not found"}, 404



from flask_login import current_user

class Section():
	def __init__(self, title, permission=None):
		self.title = title
		self.links = []

	def add_link(self, link):
		self.links.append(link)
	
	def get_link(self, title):
		for link in self.links:
			if link.title == title:
				return link

	@property
	def permissions(self):
		return [link.permission for link in self.links]

	def render_links(self):
		return "".join(
			[link.render() for link in self.links
			 if current_user.can(link.permission)]
		)

	def render(self):
		return """<div class="menu_section">
<h3>{title}</h3>
<ul class="nav side-menu">
{links}
</ul>
</div>""".format(title=self.title, links=self.render_links())


class Link():
	def __init__(self, icon, title, href="#", permission=None):
		self.children = []
		self.title = title
		self.href = href
		self.icon = icon
		self.permission = permission

	def add_child(self, child):
		self.children.append(child)

	def fa_chevron_down(self):
		if self.children:
			return '<span class="fa fa-chevron-down"></span>'
		return ''

	def render_children(self):
		return "".join([child.render() for child in self.children
						if current_user.can(child.permission)])

	def render(self):
		return """<li><a href="{href}"><i class="{icon}"></i> {title} {chevron}</a>
<ul class="nav child_menu">
{children}
</ul>
</li>""".format(icon=self.icon, title=self.title, href=self.href,
	children=self.render_children(), chevron=self.fa_chevron_down())


class ChildLink():
	def __init__(self, name, href, permission=None):
		self.name = name
		self.href = href
		self.permission = permission

	def render(self):
		return '<li><a href="{href}">{name}</a></li>'.format(
			href=self.href,
			name=self.name
		)


class PanelSidebar():
	def __init__(self):
		self.sections = []

	def add_section(self, section):
		self.sections.append(section)

	def get_section(self, title):
		for section in self.sections:
			if section.title == title:
				return section

	def render(self):
		sections = []
		for section in self.sections:
			found = False
			for perm in section.permissions:
				if current_user.can(perm):
					sections.append(section)
					break

		return "".join([section.render() for section in sections])


panel_sidebar = PanelSidebar()


from flask import current_app
from .sidebar import panel_sidebar

def get_text_direction():
	return current_app.config['PANEL_BASE_DIR']

def rightrtl_leftltr():
	return "right" if current_app.config['PANEL_BASE_DIR'] == "rtl" else "left"

def leftrtl_rightltr():
	return "left" if current_app.config['PANEL_BASE_DIR'] == "rtl" else "right"

def get_sidebar():
	return panel_sidebar.render()

def get_sitename():
	return current_app.config['SITE_NAME']

def add_globals(app):
	app.jinja_env.globals['get_sitename'] = get_sitename
	app.jinja_env.globals['get_sidebar'] = get_sidebar
	
	app.jinja_env.globals['get_text_direction'] = get_text_direction
	app.jinja_env.globals['rightrtl_leftltr'] = rightrtl_leftltr
	app.jinja_env.globals['leftrtl_rightltr'] = leftrtl_rightltr


import os
import uuid

from pathlib import Path
from flask import current_app, url_for
from werkzeug.utils import secure_filename

from ..auth.models import AbstractModelWithPermission

__all__ = ('File', 'setup_permissions')

def setup_permissions():
    File.setup_permissions()


class File(AbstractModelWithPermission):
    __slot__ = [
        'uuid', 'file_name', 'file', 'location', '_extension', '_name',
        '_url', '_abs_folder_path', '_abs_path'
    ]
    def __init__(self, location, file=None, file_name=None):
        attributes = ('_extension', '_name','_url', '_abs_folder_path', '_abs_path')
        self.uuid = uuid.uuid4()
        self.file_name = file_name or str(self.uuid)
        self.file = file
        self.location = location
        
        for attr in attributes:
            setattr(self, attr, None)

    @property
    def extension(self):
        if self._extension is None:
            self._extension = self.file.filename.rsplit('.', 1)[1].lower()
            return self._extension
        return self._extension

    @property
    def name(self):
        if self._name is None:
            self._name = "%s.%s" % (self.file_name, self.extension)
            return self._name
        return self._name

    def create_folders(self):
        if not os.path.isdir(str(self.abs_folder_path)):
            os.makedirs(str(self.abs_folder_path))

    def save(self):
        self.create_folders()
        filename = secure_filename(self.name)
        self.file.save(os.path.join(str(self.abs_folder_path), filename))

    @property
    def url(self):
        if self._url is None:
            path_format = '%s/%s'
            if self.file:
                path = path_format % (self.location, self.name)
                self._url = url_for('media.file_loader',
                    path_to_file=path)
            else:
                path = path_format % (self.location, self.file_name)
                self._url = url_for('media.file_loader',
                    path_to_file=path, _external=True)
            return self._url
        return self._url

    @property
    def abs_folder_path(self):
        if self._abs_folder_path is None:
            self._abs_folder_path = current_app.config['UPLOAD_FOLDER'] / self.location
            return self._abs_folder_path
        return self._abs_folder_path

    @property
    def abs_path(self):
        if self._abs_path is None:
            self._abs_path = self.abs_folder_path / self.name
            return self._abs_path
        return self._abs_path

    class Meta:
        permissions = (
            ('can_see_directory', 'Can see directory listing'),
            ('can_create_files', 'Can create files'),
            ('can_delete_files', 'Can delete files'),
        )


import os

from pathlib import Path
from flask_babel import lazy_gettext


PATH_TO_ROOT = Path(__file__).parent.parent


class BaseConfig(object):
    VERSION = "0.1.0"
    MINIFY = False

    SQLALCHEMY_DATABASE_URI = 'postgresql://username:password@server_address/db_name'
    DEBUG = True

    SECRET_KEY = 'GET SECRET KEY FROM ENV'

    SQLALCHEMY_TRACK_MODIFICATIONS = False

    JWT_BLACKLIST_ENABLED = True
    JWT_BLACKLIST_TOKEN_CHECKS = ['access', 'refresh']
    JWT_REFRESH_TOKEN_EXPIRES = False
    JWT_ACCESS_TOKEN_EXPIRES = False

    UPLOAD_FOLDER = Path(__file__).parent.parent / 'uploads'

    FARDEL_APP_PATH = "fardel_apps"
    ACTIVE_APPS = (
        "blog",
        "auth_address",
        "ecommerce",
    )

    SITE_NAME = lazy_gettext("Fardel")

    CACHE_TYPE = 'simple'

    PANEL_BASE_DIR = "ltr"
    BABEL_DEFAULT_LOCALE = "en"
    BABEL_TRANSLATION_DIRECTORIES = str(PATH_TO_ROOT / "translations")

    SENTRY_DSN = ""

    EMAIL_VERIFICATION_EMAIL_SENDER = os.getenv("EMAIL_VERIFICATION_EMAIL_SENDER")
    EMAIL_VERIFICATION_ACTIVE = bool(os.getenv("EMAIL_VERIFICATION_ACTIVE", default=False))

    MAIL_SERVER = os.getenv("MAIL_SERVER")
    MAIL_PORT = int(os.getenv("MAIL_PORT", default=465))
    MAIL_USERNAME = os.getenv("MAIL_USERNAME")
    MAIL_PASSWORD = os.getenv("MAIL_PASSWORD")
    MAIL_DEFAULT_SENDER = os.getenv("MAIL_DEFAULT_SENDER")



"""
Required flask extensions objects in RatSnake are created here.
"""

from flask_sqlalchemy import SQLAlchemy
from flask_jwt_extended import JWTManager
from flask_caching import Cache
from flask_login import LoginManager
from flask_babel import Babel
from flask_mail import Mail


db = SQLAlchemy()
jwt = JWTManager()
cache = Cache()
login_manager = LoginManager()
babel = Babel()
mail = Mail()



from setuptools import setup, find_packages


install_requires = [
    "flask==1.1.1",
    "flask_mail==0.9.1",
    "flask_sqlalchemy==2.3.2",
    "psycopg2-binary",
    "bcrypt==3.1.4",
    "flask-restful==0.3.6",
    "flask_migrate==2.1.1",
    "flask_jwt_extended==3.8.1",
    "flask-script==2.0.6",
    "flask_login==0.4.1",
    "flask_caching==1.4.0",
    "Flask-Babel==0.11.2",
    "raven[flask]",
]

setup(
    name='fardel',
    version='1.1.0',
    description='Complete and modular CMS',
    author='Sepehr Hamzehlouy',
    author_email='s.hamzelooy@gmail.com',
    url='https://github.com/FardelCMS/FardelCMS',
    packages=find_packages(".", exclude=["tests", "tests.*"]),
    include_package_data=True,
    package_data={
        'static': ['*.html', '*.css', '*.ttf', '*.eot', '*.svg', '*.woff', '*.woff2', '*.otf', '*.js', '*.js.map', '*.gif'],
    },
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Programming Language :: Python :: 3.7',
    ],
)



"""
All files are accessible through this url

:URL: ``/uploads/<path_to_file>``

"""


from flask import send_file, current_app

from . import mod
from .models import setup_permissions


@mod.before_app_first_request
def create_permissions():
    setup_permissions()


@mod.route('/uploads/<path:path_to_file>')
def file_loader(path_to_file):
	""" Use this function only in development mode """
	return send_file(str(current_app.config['UPLOAD_FOLDER'] / path_to_file))


from fardel.ext import db


class SeoModel(object):
	seo_title = db.Column(db.String(70))
	seo_description = db.Column(db.String(300))

	def seo_dict(self):
		return {
			'seo_title': self.seo_title,
			'seo_description': self.seo_description
		}